<?php

namespace Bydedi\Shipping\Colissimo;

use Commande;
use Csv;

/**
 * Class Expeditor
 * @package Bydedi\Shipping\Colissimo
 */
class Expeditor
{
    /**
     * @var array
     */
    static protected $config = null;

    public function __construct()
    {
        $this->checkRequired();
        $this->triggerErrorIfNotInit();

        $default = [
            'file' => ['order' => 'commandes.csv', 'tracking' => 'colis.csv'],
        ];

        self::$config = array_merge($default, self::$config);

        if (is_dir(self::$config['dir']) == false) {
            mkdir(self::$config['dir'], 0775, true);
        }
    }

    /**
     * @param array $config
     *  - trader_company_name : nom de la société qui édites les étiquettes
     *  - dir                 : dossier dans lequel on a les fichiers à intégrer à expeditor et les fichiers de retours.
     *  - file : order | tracking : nom des fichiers order => avec les commandes | tracking => avec les numéros de colis
     *  - get_order : array avec les criteres pour remonter les commandes à traiter
     */
    public static function init($config = array())
    {
        self::$config = $config;
    }

    /**
     * Check if the package is initialized.
     * If not trigger a warning error
     */
    protected function triggerErrorIfNotInit()
    {
        if (is_null(self::$config)) {
            trigger_error(sprintf("%s is not initialized", __CLASS__), E_USER_WARNING);
        }
    }

    /**
     * Vérifie que l'on a bien les classes necessaires pour utiliser toutes les fonctionnalités du package
     */
    protected function checkRequired()
    {
        if (class_exists('Commande') == false) {
            trigger_error(
                sprintf('Class "Commande" is required by %s class', __CLASS__),
                E_USER_ERROR
            );
        }

        if (class_exists('Csv') == false) {
            trigger_error(
                sprintf('Class "Csv" is required by %s class', __CLASS__),
                E_USER_ERROR
            );
        }
    }

    public function generate($file, $opts = array())
    {
        $param = array_merge(self::$config['get_order'], $opts);

        $commandes = $this->getOrder($param);

        $content = "";
        foreach ($commandes as $cmd) {
            $content.= $this->generateLine($cmd['commande_id'], $cmd) . "\r\n";
        }

        if (file_exists($file)) {
            unlink($file);
        }
        return file_put_contents($file, $content);
    }

    /**
     * @param array $param
     * @return mixed
     */
    public function getOrder($param = array())
    {
        self::triggerErrorIfNotInit();

        $commande = new Commande();
        return $commande->get($param);
    }

    /**
     * genere une ligne colis pour une commande
     *
     * @param $commandeId
     * @param array $c
     *
     * @return string
     */
    public function generateLine($commandeId, $c = array())
    {
        if (empty($c)) {
            $commande = new Commande();
            list($c) = $commande->get(array('commande_id' => $commandeId));
        }

        $prefixAddr = (!empty($c['relais_id'])) ? 'facturation' : 'livraison';

        $data = array();

        // constante
        $data[] = $this->creatData('EXP', 3);

        // ref colis pour l'expediteur | TRORDERNUMBER
        $data[] = $this->creatData($c['commande_id'], 35);

        // date expe
        $data[] = $this->creatData(date('Ymd'), 8);

        // code produit | DELIVERYMODE
        $data[] = $this->creatData($c['commande_livraison_code'], 4);

        // code expediteur
        $data[] = $this->creatData('', 6);

        // code destinataire
        $data[] = $this->creatData('', 17);

        // nom destinataire | CENAME
        $data[] = $this->creatData($c['livraison_nom'], 35);

        // Adresse 1 du destinataire : Numéro et libellé de voie | CEADRESS3
        $data[] = $this->creatData($c[$prefixAddr . '_rue'], 35);

        // Adresse 2 du destinataire : Etage, couloir, escalier, appartement | CEADRESS1
        $data[] = $this->creatData('', 35);

        // Adresse 3 du destinataire : Entrée, bâtiment, immeuble, résidence | CEADRESS2
        $data[] = $this->creatData('', 35);

        // Code postal du destinataire | CEZIPCODE
        $data[] = $this->creatData($c[$prefixAddr . '_cp'], 9);

        // Commune du destinataire | CETOWN
        $data[] = $this->creatData($c[$prefixAddr . '_ville'], 35);

        // Code pays du destinataire
        $data[] = $this->creatData($c[$prefixAddr . '_pays_code'], 3);

        // Instruction de livraison | CEDELIVERYINFORMATION
        $data[] = $this->creatData('', 70);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 12);

        // Poids
        $data[] = $this->creatData($c['commande_poids'], 15);

        // Montant CRBT
        $data[] = $this->creatData('', 15);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 3);

        // Montant ADV (assurance)
        $data[] = $this->creatData('', 15);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 3);

        // Non mécanisable : O, N
        $data[] = $this->creatData('N', 1);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 17);

        // N° téléphone (filaire)
        $data[] = $this->creatData('', 25);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 4);

        // Recommandation
        $data[] = $this->creatData('0', 1);

        // Avis de réception
        $data[] = $this->creatData('N', 1);

        // FTD (Franc de taxes et de droits)
        $data[] = $this->creatData('N', 1);

        // Instruction de non livraison (Type de retour) : 1 = Prioritaire, 2 = Economique, 3 = Abandon, Défaut : 2
        $data[] = $this->creatData('2', 1);

        // Adresse E-mail du destinataire | CEEMAIL
        $data[] = $this->creatData($c['client_email'], 75);

        // Nature envoi : 0 = Echantillon commercial, 1 = Document, 2 = Cadeau, 3 = Retour de marchandises, 4 = Envoi Commercial, 5 = Autre, Défaut : Autre
        $data[] = $this->creatData(5, 1);

        // Numéro de licence
        $data[] = $this->creatData('', 25);

        // Numéro de certificat
        $data[] = $this->creatData('', 25);

        // Numéro de facture
        $data[] = $this->creatData('', 25);

        // Reference douane
        $data[] = $this->creatData('', 4);

        // Adresse 4 du destinataire : Lieu dit ou autre mention | CEADRESS4
        $data[] = $this->creatData($c[$prefixAddr . '_rue2'], 35);

        // Numéro d’affaire
        $data[] = $this->creatData('', 35);

        // Numéro de commande | TRORDERNUMBER
        $data[] = $this->creatData($c['commande_id'], 35);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 73);

        // Civilité du destinataire : Blanc, 0 ou 1 => aucune, 2 => M., 3 => Mme, 4 => Mlle | CECIVILITY
        $data[] = $this->creatData(($c['livraison_civilite'] == 'M') ? 2 : 3, 1);

        // Prénom du destinataire | CEFIRSTNAME
        $data[] = $this->creatData($c['livraison_prenom'], 29);

        // Raison sociale du destinataire | CECOMPANYNAME
        $data[] = $this->creatData($c[$prefixAddr . '_societe'], 38);

        // Portable du destinataire | CEPHONENUMBER
        $data[] = $this->creatData($c['livraison_telephone'], 10);

        // Code porte 2 de l’adresse du destinataire | CEDOORCODE1
        $data[] = $this->creatData('', 8);

        // Code porte 1 de l’adresse du destinataire | CEDOORCODE2
        $data[] = $this->creatData('', 8);

        // Interphone de l’adresse du destinataire | CEENTRYPHONE
        $data[] = $this->creatData('', 30);

        // Code point de retrait | PRID
        $data[] = $this->creatData($c['relais_id'], 6);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 1);

        // Nom commercial de l’expéditeur | TRADERCOMPANYNAME
        $data[] = $this->creatData(self::$config['trader_company_name'], 38);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 30);

        // Code avoir / promotion
        $data[] = $this->creatData('', 15);

        // Filler (zone non utilisée)
        $data[] = $this->creatData('', 75);

        // Code réseau : Code réseau du point de retrait R12 = point en Belgique Vide = point en France | CODERESEAU
        $data[] = $this->creatData(($c['livraison_pays_code'] == 'BE') ? 'R12' : '', 3);

        // Portable international : Pour le produit SO Belgique. Au format +324XXXXXXXX | CEPHONENUMBER
        $data[] = $this->creatData('', 20);

        $data[] = $this->creatData('', 35);
        $data[] = $this->creatData('', 35);
        $data[] = $this->creatData('', 35);
        $data[] = $this->creatData('', 35);
        $data[] = $this->creatData('', 35);
        $data[] = $this->creatData('', 5);

        $data[] = $this->creatData('', 35);
        $data[] = $this->creatData('', 4);
        $data[] = $this->creatData('', 4);
        $data[] = $this->creatData('', 4);
        $data[] = $this->creatData('', 2);

        return implode('', $data);

    }

    public function track($file)
    {
        $commande = new Commande();
        $csv = new Csv($file);
        $csv->loadData();

        $i = 0;
        while ($data = $csv->getNextLine()) {
            $i++;
            if ($i == 1) {
                // on ne traite pas la 1ere ligne : entete col
                continue;
            }

            // on expedie la commande
            $commande->changeStatut($data[0], Commande::STATUT_EXPEDIE, true, array('colis' => $data[1]));
        }

        // on supprime le fichier traité
        unlink($file);
    }

    /**
     * @param array $opts
     * @return string
     */
    public function matin($opts = array())
    {
        if (file_exists(self::$config['dir'] . "/" . self::$config['file']['tracking'])) {
            return "Impossible d'&eacute;xecuter le script Expeditor Matin.<br/>
                    Vous devez auparavant &eacute;xecut&eacute; le script Expeditor Soir.<br/>";
        } else {
            $this->generate(self::$config['dir'] . "/" . self::$config['file']['order'], $opts);
            return "Le script Expeditor Matin s'est &eacute;xecut&eacute; avec succ&egrave;s.<br/>
                    Vous pouvez cliquer sur l'icone Matin se situant sur le bureau de votre ordinateur.<br/>";
        }
    }

    /**
     * @return string
     */
    public function soir()
    {
        if (!file_exists(self::$config['dir'] . "/" . self::$config['file']['tracking'])) {
            return "Impossible d'&eacute;xecuter le script Expeditor Soir.<br/>
            Vous devez auparavant cliquer sur l'icone Soir se situant sur le bureau de votre ordinateur.<br/>";
        } else {

            $this->track(self::$config['dir'] . "/" . self::$config['file']['tracking']);

            return "Le script Expeditor Soir s'est &eacute;xecut&eacute; avec succ&egrave;s.<br/>
                    <a href='#' onclick='window.reload()'>Cliquez ici pour visualiser les changements</a><br/>";
        }
    }

    /**
     * @param $data
     * @param null $length
     * @return string
     */
    protected function creatData($data, $length = null)
    {
        if (!is_null($length)) {
            $lengthData = mb_strlen($data);
            if ($lengthData > $length) {
                $data = substr($data, 0, $length);
            } elseif ($lengthData < $length) {
                $data = str_pad($data, $length, " ", STR_PAD_RIGHT);
            }
        }

        return $data;
    }
}
